Attribute VB_Name = "modDDS"
Option Explicit

Private Type ddscolorkey '8 bytes
    dw1 As Long
    dw2 As Long
End Type

Private Type ddspixelformat '32 bytes
    dwSize As Long
    dwFlags As Long
    dwFourCC As String * 4
    u1 As Long
    u2 As Long
    u3 As Long
    u4 As Long
    u5 As Long
End Type

Private Type ddscaps '16 bytes
    dwCaps As Long
    dwCaps2 As Long
    dwCaps3 As Long
    dwCaps4 As Long
End Type

Private Type ddsheader '124 bytes
    dwSize As Long
    dwFlags As Long
    dwWidth As Long
    dwHeight As Long
    dwLinearSize As Long
    dwBackBufferCount As Long
    dwMipMapCount As Long
    dwAlphaBitDepth As Long
    dwReserved As Long
    lpSurface As Long
    ddckCKDestOverlay As ddscolorkey
    ddckCKDestBlt As ddscolorkey
    ddckCKSrcOverlay As ddscolorkey
    ddckCKSrcBlt As ddscolorkey
    ddpfPixelFormat As ddspixelformat
    ddscaps As ddscaps
    dwTextureStage As Long
End Type


'parses DDS data to step to end of chunk
' note: fourCC has already been read at this point
Public Function ParseDDS(ByVal ff As Integer) As Long
    On Error GoTo errorhandler
    
    'get current location
    Dim start As Long
    start = Loc(ff)
    
    'read surface descriptor
    Dim ddsd As ddsheader
    Get #ff, , ddsd
    
    Echo ">>> format: " & ddsd.ddpfPixelFormat.dwFourCC
    
    'detect DDS type
    Dim fcode As String
    fcode = Left(ddsd.ddpfPixelFormat.dwFourCC, 4)
    If Left(fcode, 3) <> "DXT" Then fcode = ""
    Echo ">>> ddsd.ddpfPixelFormat.dwFourCC: " & fcode
    
    'determine some stuff
    Dim comp As Boolean
    Dim blocksize As Long
    Dim mipmapfactor As Long
    Dim compratio As Long
    Select Case fcode
    Case "" 'uncompressed
        blocksize = 0
        mipmapfactor = 1
        comp = False
        compratio = 1
    Case "DXT1" 'DXT1 compression ratio is 8:1
        blocksize = 8
        mipmapfactor = 2
        comp = True
        compratio = 8
    Case "DXT3" 'DXT3 compression ratio is 4:1
        blocksize = 16
        mipmapfactor = 4
        comp = True
        compratio = 4
    Case "DXT5" 'DXT5 compression ratio is 4:1
        blocksize = 16
        mipmapfactor = 4
        comp = True
        compratio = 4
    Case Else
        Echo "DSS format not supported."
        Exit Function
    End Select
    
    Dim mipmaps As Long
    mipmaps = ddsd.dwMipMapCount
    If ddsd.dwMipMapCount = 0 Then 'happens when uncompressed
        mipmaps = 1
    Else
        mipmaps = ddsd.dwMipMapCount
    End If
    
    Dim w As Long
    Dim h As Long
    w = ddsd.dwWidth
    h = ddsd.dwHeight
    
    'compute size
    Dim size As Long
    Dim i As Long
    For i = 0 To mipmaps - 1
        size = size + (w * h * 4) / compratio
        w = w / 2
        h = h / 2
    Next i
    
    'success
    ParseDDS = size
    Exit Function
    
    'success
    Exit Function
errorhandler:
    Echo ">>> ParseDDS:" & Err.Description
End Function

